/*
 * Sentinella
 * http://sourceforge.net/projects/sentinella/
 * Copyright (c) 2009 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <iostream>
#include <errno.h>
#include <QtGlobal>
#include <KApplication>
#include <KAboutData>
#include <KCmdLineArgs>
#include "MainWindow.h"

int main(int argc, char **argv) {
	KAboutData aboutData("Sentinella", NULL, ki18n("Sentinella"), "0.6.1", ki18n(
			"Action launcher based on system activity"), KAboutData::License_GPL_V2, ki18n(
			"Copyright (c) 2009, 2010 Carlos Olmedo Escobar"), ki18n(""),
			"http://sentinella.sourceforge.net/",
			"http://sourceforge.net/tracker/?func=add&group_id=277778&atid=1179456");
	aboutData.addAuthor(
			ki18n("Carlos Olmedo Escobar"),
			ki18n(
					"Project leader. Audio files released under the license: http://creativecommons.org/licenses/by-sa/3.0/"),
			"carlos.olmedo.e@gmail.com");
	aboutData.addCredit(ki18n("Damon John Hoxworth"), ki18n("Logo and web designer"),
			"damon@solutionarts.net", "http://solutionarts.net/");
	aboutData.addCredit(ki18n("Dave Summitt"), ki18n("Correction of the english translation."),
			"dave@summitt.net");
	KCmdLineArgs::init(argc, argv, &aboutData);

	KCmdLineOptions options;
	options.add("s").add("start", ki18n("The condition will be ran immediately."));

	options.add(":", ki18n("Condition options:"));
	options.add("cpu <percentage>", ki18n(
			"CPU load measurement. Use in combination with --during, --lower or --higher."));
	options.add("memory <megabytes>", ki18n(
			"Memory usage in megabytes. Use in combination with --during, --lower or --higher."));
	options.add(
			"network <kb>",
			ki18n(
					"Network traffic in kilobytes/second. Use in combination with --interface, --during and --lower or --higher."));
	options.add("date-time <value>", ki18n(
			"Current date and time condition. The format should be: YYYY-MM-DDTHH:MM:SS"));
	options.add("program-dies <pid>", ki18n(
			"The condition will be met when the process of id <pid> dies."));
	options.add("h").add("higher", ki18n(
			"The condition will be met if the system's activity is higher than your chosen value."));
	options.add("l").add("lower", ki18n(
			"The condition will be met if the system's activity is lower than your chosen value."));
	options.add("d").add("during <time>", ki18n(
			"The time during the condition must be met to throw the action. Format: HH:MM:SS."));
	options.add("i").add(
			"interface <name>",
			ki18n(
					"Sets the name of the network interface to monitor when using the --network parameter."));

	options.add(":", ki18n("Action options:"));
	options.add("shutdown <force>", ki18n(
			"Shutdown the computer when the condition is met. <force> values are 'yes' or 'no'."));
	options.add(
			"reboot <force>",
			ki18n(
					"Reboot the computer when the condition is met. Possible values for <force> are 'yes' or 'no'."));
	options.add(
			"sleep <mode>",
			ki18n(
					"Place computer in sleep mode when the condition is met. Possible values for <force> are 'Standby', 'Suspend' or 'Hibernate'."));
	options.add("terminate-program <pid>", ki18n(
			"It will try to terminate the program specified by <pid>."));
	KCmdLineArgs::addCmdLineOptions(options);

	KCmdLineArgs* parsedArgs = KCmdLineArgs::parsedArgs(); // Validating condition parameters
	QByteArray args[] = { "cpu", "memory", "network", "date-time", "program-dies" };
	short i, chosenCondition;
	chosenCondition = 0;
	for (i = 0; i < 5; i++) {
		if (parsedArgs->isSet(args[i])) {
			if (!chosenCondition)
				chosenCondition = i + 1;
			else {
				std::cout << qPrintable(
						ki18n("Sorry. You can choose only one condition.").toString()) << std::endl;
				return EINVAL;
			}
		}
	}
	if ((!chosenCondition || chosenCondition > 3) && (parsedArgs->isSet("higher")
			|| parsedArgs->isSet("lower") || parsedArgs->isSet("during"))) {
		std::cout
				<< qPrintable(
						ki18n(
								"Sorry. Parameters --higher, --lower and --during are only valid in conjunction with --cpu, --memory and --network conditions.").toString())
				<< std::endl;
		return EINVAL;
	}

	args[0] = "shutdown"; // Validating action parameters
	args[1] = "reboot";
	args[2] = "sleep";
	args[3] = "terminate-program";
	short chosenAction = 0;
	for (i = 0; i < 4; i++) {
		if (parsedArgs->isSet(args[i])) {
			if (!chosenAction)
				chosenAction = i + 1;
			else {
				std::cout << qPrintable(ki18n("Sorry. You can choose only one action.").toString())
						<< std::endl;
				return EINVAL;
			}
		}
	}
	if (chosenAction == 4) // "terminate-program" is the sixth action in GUI, so..
		chosenAction = 6;

	KApplication app;
	MainWindow* mainWindow = new MainWindow(aboutData, parsedArgs, chosenCondition, chosenAction);

	mainWindow->show();
	return app.exec();
}
